@@ -10,14 +10,18 @@ from twisted.application.internet import TimerService
 from twisted.python import log
 
 import allmydata
-from allmydata.Crypto.Util.number import bytes_to_long
 from allmydata.storage import StorageServer
 from allmydata.upload import Uploader
 from allmydata.download import Downloader
+from allmydata.checker import Checker
 from allmydata.control import ControlServer
 from allmydata.introducer import IntroducerClient
-from allmydata.vdrive import VirtualDrive
 from allmydata.util import hashutil, idlib, testutil
+from allmydata.filenode import FileNode
+from allmydata.dirnode import NewDirectoryNode
+from allmydata.mutable import MutableFileNode
+from allmydata.interfaces import IURI, INewDirectoryURI, \
+     IReadonlyNewDirectoryURI, IFileURI, IMutableFileURI
 
 class Client(node.Node, Referenceable, testutil.PollMixin):
     implements(RIClient)
@@ -34,15 +38,12 @@ class Client(node.Node, Referenceable, testutil.PollMixin):
         self.logSource="Client"
         self.my_furl = None
         self.introducer_client = None
-        self.init_secret()
+        self.init_lease_secret()
         self.init_storage()
         self.init_options()
         self.add_service(Uploader())
         self.add_service(Downloader())
-        self.add_service(VirtualDrive())
-        webport = self.get_config("webport")
-        if webport:
-            self.init_web(webport) # strports string
+        self.add_service(Checker())
 
         self.introducer_furl = self.get_config("introducer.furl", required=True)
 
@@ -54,12 +55,15 @@ class Client(node.Node, Referenceable, testutil.PollMixin):
             hotline = TimerService(1.0, self._check_hotline, hotline_file)
             hotline.setServiceParent(self)
 
-    def init_secret(self):
+        webport = self.get_config("webport")
+        if webport:
+            self.init_web(webport) # strports string
+
+    def init_lease_secret(self):
         def make_secret():
-            return idlib.b2a(os.urandom(16)) + "\n"
-        secret_s = self.get_or_create_config("secret", make_secret,
-                                             filemode=0600)
-        self._secret = idlib.a2b(secret_s)
+            return idlib.b2a(os.urandom(hashutil.CRYPTO_VAL_SIZE)) + "\n"
+        secret_s = self.get_or_create_private_config("secret", make_secret)
+        self._lease_secret = idlib.a2b(secret_s)
 
     def init_storage(self):
         storedir = os.path.join(self.basedir, self.STOREDIR)
@@ -90,18 +94,14 @@ class Client(node.Node, Referenceable, testutil.PollMixin):
             self.push_to_ourselves = True
 
     def init_web(self, webport):
+        self.log("init_web(webport=%s)", args=(webport,))
+
         from allmydata.webish import WebishServer
-        # this must be called after the VirtualDrive is attached
-        ws = WebishServer(webport)
+        nodeurl_path = os.path.join(self.basedir, "node.url")
+        ws = WebishServer(webport, nodeurl_path)
         if self.get_config("webport_allow_localfile") is not None:
             ws.allow_local_access(True)
         self.add_service(ws)
-        vd = self.getServiceNamed("vdrive")
-        startfile = os.path.join(self.basedir, "start.html")
-        nodeurl_file = os.path.join(self.basedir, "node.url")
-        d = vd.when_private_root_available()
-        d.addCallback(ws.create_start_html, startfile, nodeurl_file)
-
 
     def _check_hotline(self, hotline_file):
         if os.path.exists(hotline_file):
@@ -116,6 +116,7 @@ class Client(node.Node, Referenceable, testutil.PollMixin):
 
     def tub_ready(self):
         self.log("tub_ready")
+        node.Node.tub_ready(self)
 
         # we use separate get_config/write_config here because we want to
         # update the connection hints each time.
@@ -138,10 +139,7 @@ class Client(node.Node, Referenceable, testutil.PollMixin):
         c = ControlServer()
         c.setServiceParent(self)
         control_url = self.tub.registerReference(c)
-        control_furl_file = os.path.join(self.basedir, "control.furl")
-        open(control_furl_file, "w").write(control_url + "\n")
-        os.chmod(control_furl_file, 0600)
-
+        self.write_private_config("control.furl", control_url + "\n")
 
     def remote_get_versions(self):
         return str(allmydata.__version__), str(self.OLDEST_SUPPORTED_VERSION)
@@ -169,7 +167,7 @@ class Client(node.Node, Referenceable, testutil.PollMixin):
             if not include_myself and peerid == self.nodeid:
                 self.log("get_permuted_peers: removing myself from the list")
                 continue
-            permuted = bytes_to_long(sha.new(key + peerid).digest())
+            permuted = sha.new(key + peerid).digest()
             results.append((permuted, peerid, connection))
         results.sort()
         return results
@@ -188,10 +186,10 @@ class Client(node.Node, Referenceable, testutil.PollMixin):
         return False
 
     def get_renewal_secret(self):
-        return hashutil.my_renewal_secret_hash(self._secret)
+        return hashutil.my_renewal_secret_hash(self._lease_secret)
 
     def get_cancel_secret(self):
-        return hashutil.my_cancel_secret_hash(self._secret)
+        return hashutil.my_cancel_secret_hash(self._lease_secret)
 
     def debug_wait_for_client_connections(self, num_clients):
         """Return a Deferred that fires (with None) when we have connections
@@ -205,3 +203,39 @@ class Client(node.Node, Referenceable, testutil.PollMixin):
         d.addCallback(lambda res: None)
         return d
 
+
+    # these four methods are the primitives for creating filenodes and
+    # dirnodes. The first takes a URI and produces a filenode or (new-style)
+    # dirnode. The other three create brand-new filenodes/dirnodes.
+
+    def create_node_from_uri(self, u):
+        # this returns synchronously.
+        u = IURI(u)
+        if IReadonlyNewDirectoryURI.providedBy(u):
+            # new-style read-only dirnodes
+            return NewDirectoryNode(self).init_from_uri(u)
+        if INewDirectoryURI.providedBy(u):
+            # new-style dirnodes
+            return NewDirectoryNode(self).init_from_uri(u)
+        if IFileURI.providedBy(u):
+            # CHK
+            return FileNode(u, self)
+        assert IMutableFileURI.providedBy(u), u
+        return MutableFileNode(self).init_from_uri(u)
+
+    def create_empty_dirnode(self, wait_for_numpeers=None):
+        n = NewDirectoryNode(self)
+        d = n.create(wait_for_numpeers=wait_for_numpeers)
+        d.addCallback(lambda res: n)
+        return d
+
+    def create_mutable_file(self, contents="", wait_for_numpeers=None):
+        n = MutableFileNode(self)
+        d = n.create(contents, wait_for_numpeers=wait_for_numpeers)
+        d.addCallback(lambda res: n)
+        return d
+
+    def upload(self, uploadable, wait_for_numpeers=None):
+        uploader = self.getServiceNamed("uploader")
+        return uploader.upload(uploadable, wait_for_numpeers=wait_for_numpeers)
+
