@@ -13,7 +13,7 @@ Nodeid = StringConstraint(maxLength=20,
 FURL = StringConstraint(1000)
 StorageIndex = StringConstraint(16)
 URI = StringConstraint(300) # kind of arbitrary
-DirnodeURI = StringConstraint(300, regexp=r'^URI:DIR(-RO)?:pb://[a-z0-9]+@[^/]+/[^:]+:[a-z0-9]+$')
+
 MAX_BUCKETS = 200  # per peer
 
 # MAX_SEGMENT_SIZE in encode.py is 1 MiB (this constraint allows k = 1)
@@ -82,6 +82,20 @@ class RIBucketReader(RemoteInterface):
     def read(offset=int, length=int):
         return ShareData
 
+TestVector = ListOf(TupleOf(int, int, str, str))
+# elements are (offset, length, operator, specimen)
+# operator is one of "lt, le, eq, ne, ge, gt"
+# nop always passes and is used to fetch data while writing.
+# you should use length==len(specimen) for everything except nop
+DataVector = ListOf(TupleOf(int, ShareData))
+# (offset, data). This limits us to 30 writes of 1MiB each per call
+TestAndWriteVectorsForShares = DictOf(int,
+                                      TupleOf(TestVector,
+                                              DataVector,
+                                              ChoiceOf(None, int))) # new_length
+ReadVector = ListOf(TupleOf(int, int))
+ReadData = ListOf(ShareData)
+# returns data[offset:offset+length] for each element of TestVector
 
 class RIStorageServer(RemoteInterface):
     def allocate_buckets(storage_index=StorageIndex,
@@ -126,6 +140,90 @@ class RIStorageServer(RemoteInterface):
         return DictOf(int, RIBucketReader, maxKeys=MAX_BUCKETS)
 
 
+
+    def slot_readv(storage_index=StorageIndex,
+                   shares=ListOf(int), readv=ReadVector):
+        """Read a vector from the numbered shares associated with the given
+        storage index. An empty shares list means to return data from all
+        known shares. Returns a dictionary with one key per share."""
+        return DictOf(int, ReadData) # shnum -> results
+
+    def slot_testv_and_readv_and_writev(storage_index=StorageIndex,
+                                        secrets=TupleOf(Hash, Hash, Hash),
+                                        tw_vectors=TestAndWriteVectorsForShares,
+                                        r_vector=ReadVector,
+                                        ):
+        """General-purpose test-and-set operation for mutable slots. Perform
+        a bunch of comparisons against the existing shares. If they all pass,
+        then apply a bunch of write vectors to those shares. Then use the
+        read vectors to extract data from all the shares and return the data.
+
+        This method is, um, large. The goal is to allow clients to update all
+        the shares associated with a mutable file in a single round trip.
+
+        @param storage_index: the index of the bucket to be created or
+                              increfed.
+        @param write_enabler: a secret that is stored along with the slot.
+                              Writes are accepted from any caller who can
+                              present the matching secret. A different secret
+                              should be used for each slot*server pair.
+        @param renew_secret: This is the secret used to protect bucket refresh
+                             This secret is generated by the client and
+                             stored for later comparison by the server. Each
+                             server is given a different secret.
+        @param cancel_secret: Like renew_secret, but protects bucket decref.
+
+        The 'secrets' argument is a tuple of (write_enabler, renew_secret,
+        cancel_secret). The first is required to perform any write. The
+        latter two are used when allocating new shares. To simply acquire a
+        new lease on existing shares, use an empty testv and an empty writev.
+
+        Each share can have a separate test vector (i.e. a list of
+        comparisons to perform). If all vectors for all shares pass, then all
+        writes for all shares are recorded. Each comparison is a 4-tuple of
+        (offset, length, operator, specimen), which effectively does a bool(
+        (read(offset, length)) OPERATOR specimen ) and only performs the
+        write if all these evaluate to True. Basic test-and-set uses 'eq'.
+        Write-if-newer uses a seqnum and (offset, length, 'lt', specimen).
+        Write-if-same-or-newer uses 'le'.
+
+        Reads from the end of the container are truncated, and missing shares
+        behave like empty ones, so to assert that a share doesn't exist (for
+        use when creating a new share), use (0, 1, 'eq', '').
+
+        The write vector will be applied to the given share, expanding it if
+        necessary. A write vector applied to a share number that did not
+        exist previously will cause that share to be created.
+
+        Each write vector is accompanied by a 'new_length' argument. If
+        new_length is not None, use it to set the size of the container. This
+        can be used to pre-allocate space for a series of upcoming writes, or
+        truncate existing data. If the container is growing, new_length will
+        be applied before datav. If the container is shrinking, it will be
+        applied afterwards.
+
+        The read vector is used to extract data from all known shares,
+        *before* any writes have been applied. The same vector is used for
+        all shares. This captures the state that was tested by the test
+        vector.
+
+        This method returns two values: a boolean and a dict. The boolean is
+        True if the write vectors were applied, False if not. The dict is
+        keyed by share number, and each value contains a list of strings, one
+        for each element of the read vector.
+
+        If the write_enabler is wrong, this will raise BadWriteEnablerError.
+        To enable share migration, the exception will have the nodeid used
+        for the old write enabler embedded in it, in the following string::
+
+         The write enabler was recorded by nodeid '%s'.
+
+        Note that the nodeid here is encoded using the same base32 encoding
+        used by Foolscap and allmydata.util.idlib.nodeid_b2a().
+
+        """
+        return TupleOf(bool, DictOf(int, ReadData))
+
 class IStorageBucketWriter(Interface):
     def put_block(segmentnum=int, data=ShareData):
         """@param data: For most segments, this data will be 'blocksize'
@@ -147,7 +245,7 @@ class IStorageBucketWriter(Interface):
         """
         @return: a Deferred that fires (with None) when the operation completes
         """
-        
+
     def put_share_hashes(sharehashes=ListOf(TupleOf(int, Hash),
                                             maxLength=2**20)):
         """
@@ -217,81 +315,12 @@ class IStorageBucketReader(Interface):
 
 # hm, we need a solution for forward references in schemas
 from foolscap.schema import Any
-RIMutableDirectoryNode_ = Any() # TODO: how can we avoid this?
 
 FileNode_ = Any() # TODO: foolscap needs constraints on copyables
 DirectoryNode_ = Any() # TODO: same
 AnyNode_ = ChoiceOf(FileNode_, DirectoryNode_)
 EncryptedThing = str
 
-class RIVirtualDriveServer(RemoteInterface):
-    def get_public_root_uri():
-        """Obtain the URI for this server's global publically-writable root
-        directory. This returns a read-write directory URI.
-
-        If this vdrive server does not offer a public root, this will
-        raise an exception."""
-        return DirnodeURI
-
-    def create_directory(index=Hash, write_enabler=Hash):
-        """Create a new (empty) directory, unattached to anything else.
-
-        This returns the same index that was passed in.
-        """
-        return Hash
-
-    def get(index=Hash, key=Hash):
-        """Retrieve a named child of the given directory. 'index' specifies
-        which directory is being accessed, and is generally the hash of the
-        read key. 'key' is the hash of the read key and the child name.
-
-        This operation returns a pair of encrypted strings. The first string
-        is meant to be decrypted by the Write Key and provides read-write
-        access to the child. If this directory holds read-only access to the
-        child, this first string will be an empty string. The second string
-        is meant to be decrypted by the Read Key and provides read-only
-        access to the child.
-
-        When the child is a read-write directory, the encrypted URI:DIR-RO
-        will be in the read slot, and the encrypted URI:DIR will be in the
-        write slot. When the child is a read-only directory, the encrypted
-        URI:DIR-RO will be in the read slot and the write slot will be empty.
-        When the child is a CHK file, the encrypted URI:CHK will be in the
-        read slot, and the write slot will be empty.
-
-        This might raise IndexError if there is no child by the desired name.
-        """
-        return (EncryptedThing, EncryptedThing)
-
-    def list(index=Hash):
-        """List the contents of a directory.
-
-        This returns a list of (NAME, WRITE, READ) tuples. Each value is an
-        encrypted string (although the WRITE value may sometimes be an empty
-        string).
-
-        NAME: the child name, encrypted with the Read Key
-        WRITE: the child write URI, encrypted with the Write Key, or an
-               empty string if this child is read-only
-        READ: the child read URI, encrypted with the Read Key
-        """
-        return ListOf((EncryptedThing, EncryptedThing, EncryptedThing),
-                      maxLength=1000,
-                      )
-
-    def set(index=Hash, write_enabler=Hash, key=Hash,
-            name=EncryptedThing, write=EncryptedThing, read=EncryptedThing):
-        """Set a child object. I will replace any existing child of the same
-        name.
-        """
-
-    def delete(index=Hash, write_enabler=Hash, key=Hash):
-        """Delete a specific child.
-
-        This uses the hashed key to locate a specific child, and deletes it.
-        """
-
-
 class IURI(Interface):
     def init_from_string(uri):
         """Accept a string (as created by my to_string() method) and populate
@@ -311,6 +340,25 @@ class IURI(Interface):
         """Return another IURI instance, which represents a read-only form of
         this one. If is_readonly() is True, this returns self."""
 
+    def get_verifier():
+        """Return an instance that provides IVerifierURI, which can be used
+        to check on the availability of the file or directory, without
+        providing enough capabilities to actually read or modify the
+        contents. This may return None if the file does not need checking or
+        verification (e.g. LIT URIs).
+        """
+
+    def to_string():
+        """Return a string of printable ASCII characters, suitable for
+        passing into init_from_string."""
+
+class IVerifierURI(Interface):
+    def init_from_string(uri):
+        """Accept a string (as created by my to_string() method) and populate
+        this instance with its data. I am not normally called directly,
+        please use the module-level uri.from_string() function to convert
+        arbitrary URI strings into IURI-providing instances."""
+
     def to_string():
         """Return a string of printable ASCII characters, suitable for
         passing into init_from_string."""
@@ -318,46 +366,118 @@ class IURI(Interface):
 class IDirnodeURI(Interface):
     """I am a URI which represents a dirnode."""
 
+
 class IFileURI(Interface):
     """I am a URI which represents a filenode."""
     def get_size():
         """Return the length (in bytes) of the file that I represent."""
 
+class IMutableFileURI(Interface):
+    """I am a URI which represents a mutable filenode."""
+    pass
+class INewDirectoryURI(Interface):
+    pass
+class IReadonlyNewDirectoryURI(Interface):
+    pass
 
-class IFileNode(Interface):
+
+class IFilesystemNode(Interface):
+    def get_uri():
+        """
+        Return the URI that can be used by others to get access to this
+        node. If this node is read-only, the URI will only offer read-only
+        access. If this node is read-write, the URI will offer read-write
+        access.
+
+        If you have read-write access to a node and wish to share merely
+        read-only access with others, use get_readonly_uri().
+        """
+
+    def get_readonly_uri():
+        """Return the directory URI that can be used by others to get
+        read-only access to this directory node. The result is a read-only
+        URI, regardless of whether this dirnode is read-only or read-write.
+
+        If you have merely read-only access to this dirnode,
+        get_readonly_uri() will return the same thing as get_uri().
+        """
+
+    def get_verifier():
+        """Return an IVerifierURI instance that represents the
+        'verifiy/refresh capability' for this node. The holder of this
+        capability will be able to renew the lease for this node, protecting
+        it from garbage-collection. They will also be able to ask a server if
+        it holds a share for the file or directory.
+        """
+
+    def check():
+        """Perform a file check. See IChecker.check for details."""
+
+    def is_readonly():
+        """Return True if this reference provides mutable access to the given
+        file or directory (i.e. if you can modify it), or False if not. Note
+        that even if this reference is read-only, someone else may hold a
+        read-write reference to it."""
+
+    def is_mutable():
+        """Return True if this file or directory is mutable (by *somebody*,
+        not necessarily you), False if it is is immutable. Note that a file
+        might be mutable overall, but your reference to it might be
+        read-only. On the other hand, all references to an immutable file
+        will be read-only; there are no read-write references to an immutable
+        file.
+        """
+
+class IMutableFilesystemNode(IFilesystemNode):
+    pass
+
+class IFileNode(IFilesystemNode):
     def download(target):
         """Download the file's contents to a given IDownloadTarget"""
+
     def download_to_data():
         """Download the file's contents. Return a Deferred that fires
         with those contents."""
 
-    def get_uri():
-        """Return the URI that can be used by others to get access to this
-        file.
-        """
     def get_size():
         """Return the length (in bytes) of the data this node represents."""
 
-    def get_refresh_capability():
-        """Return a string that represents the 'refresh capability' for this
-        node. The holder of this capability will be able to renew the lease
-        for this node, protecting it from garbage-collection.
+class IMutableFileNode(IFileNode, IMutableFilesystemNode):
+    def download_to_data():
+        """Download the file's contents. Return a Deferred that fires with
+        those contents. If there are multiple retrievable versions in the
+        grid (because you failed to avoid simultaneous writes, see
+        docs/mutable.txt), this will return the first version that it can
+        reconstruct, and will silently ignore the others. In the future, a
+        more advanced API will signal and provide access to the multiple
+        heads."""
+
+    def replace(newdata, wait_for_numpeers=None):
+        """Replace the old contents with the new data. Returns a Deferred
+        that fires (with None) when the operation is complete.
+
+        If the node detects that there are multiple outstanding versions of
+        the file, this will raise ConsistencyError, and may leave the
+        distributed file in an unusual state (the node will try to ensure
+        that at least one version of the file remains retrievable, but it may
+        or may not be the one you just tried to upload). You should respond
+        to this by downloading the current contents of the file and retrying
+        the replace() operation.
         """
 
-class IDirectoryNode(Interface):
-    def is_mutable():
-        """Return True if this directory is mutable, False if it is read-only.
+    def get_writekey():
+        """Return this filenode's writekey, or None if the node does not have
+        write-capability. This may be used to assist with data structures
+        that need to make certain data available only to writers, such as the
+        read-write child caps in dirnodes. The recommended process is to have
+        reader-visible data be submitted to the filenode in the clear (where
+        it will be encrypted by the filenode using the readkey), but encrypt
+        writer-visible data using this writekey.
         """
 
+class IDirectoryNode(IMutableFilesystemNode):
     def get_uri():
-        """Return the directory URI that can be used by others to get access
-        to this directory node. If this node is read-only, the URI will only
-        offer read-only access. If this node is read-write, the URI will
-        offer read-write acess.
-
-        If you have read-write access to a directory and wish to share merely
-        read-only access with others, use get_immutable_uri().
-
+        """
         The dirnode ('1') URI returned by this method can be used in
         set_uri() on a different directory ('2') to 'mount' a reference to
         this directory ('1') under the other ('2'). This URI is just a
@@ -365,24 +485,20 @@ class IDirectoryNode(Interface):
         protocol.
         """
 
-    def get_immutable_uri():
-        """Return the directory URI that can be used by others to get
-        read-only access to this directory node. The result is a read-only
-        URI, regardless of whether this dirnode is read-only or read-write.
-
-        If you have merely read-only access to this dirnode,
-        get_immutable_uri() will return the same thing as get_uri().
+    def get_readonly_uri():
         """
-
-    def get_refresh_capability():
-        """Return a string that represents the 'refresh capability' for this
-        node. The holder of this capability will be able to renew the lease
-        for this node, protecting it from garbage-collection.
+        The dirnode ('1') URI returned by this method can be used in
+        set_uri() on a different directory ('2') to 'mount' a reference to
+        this directory ('1') under the other ('2'). This URI is just a
+        string, so it can be passed around through email or other out-of-band
+        protocol.
         """
 
     def list():
         """I return a Deferred that fires with a dictionary mapping child
-        name to an IFileNode or IDirectoryNode."""
+        name to (node, metadata_dict) tuples, in which 'node' is either an
+        IFileNode or IDirectoryNode, and 'metadata_dict' is a dictionary of
+        metadata."""
 
     def has_child(name):
         """I return a Deferred that fires with a boolean, True if there
@@ -414,6 +530,13 @@ class IDirectoryNode(Interface):
         If this directory node is read-only, the Deferred will errback with a
         NotMutableError."""
 
+    def set_uris(entries):
+        """Add multiple (name, child_uri) pairs to a directory node. Returns
+        a Deferred that fires (with None) when the operation finishes. This
+        is equivalent to calling set_uri() multiple times, but is much more
+        efficient.
+        """
+
     def set_node(name, child):
         """I add a child at the specific name. I return a Deferred that fires
         when the operation finishes. This Deferred will fire with the child
@@ -423,6 +546,13 @@ class IDirectoryNode(Interface):
         If this directory node is read-only, the Deferred will errback with a
         NotMutableError."""
 
+    def set_nodes(entries):
+        """Add multiple (name, child_node) pairs to a directory node. Returns
+        a Deferred that fires (with None) when the operation finishes. This
+        is equivalent to calling set_node() multiple times, but is much more
+        efficient."""
+
+
     def add_file(name, uploadable):
         """I upload a file (using the given IUploadable), then attach the
         resulting FileNode to the directory at the given name. I return a
@@ -444,8 +574,8 @@ class IDirectoryNode(Interface):
         Deferred that fires when the operation finishes."""
 
     def build_manifest():
-        """Return a set of refresh-capabilities for all nodes (directories
-        and files) reachable from this one."""
+        """Return a frozenset of verifier-capability strings for all nodes
+        (directories and files) reachable from this one."""
 
 class ICodecEncoder(Interface):
     def set_params(data_size, required_shares, max_shares):
@@ -546,7 +676,7 @@ class ICodecEncoder(Interface):
         """
 
     def encode(inshares, desired_share_ids=None):
-        """Encode some data. This may be called multiple times. Each call is 
+        """Encode some data. This may be called multiple times. Each call is
         independent.
 
         inshares is a sequence of length required_shares, containing buffers
@@ -589,9 +719,9 @@ class ICodecEncoder(Interface):
         integer, exactly as passed into 'desired_share_ids' (or
         range(max_shares), if desired_share_ids was not provided).
 
-        The shares and their corresponding shareids are required to be kept 
-        together during storage and retrieval. Specifically, the share data is 
-        useless by itself: the decoder needs to be told which share is which 
+        The shares and their corresponding shareids are required to be kept
+        together during storage and retrieval. Specifically, the share data is
+        useless by itself: the decoder needs to be told which share is which
         by providing it with both the shareid and the actual share data.
 
         This function will allocate an amount of memory roughly equal to::
@@ -953,7 +1083,7 @@ class IUploadable(Interface):
         closed."""
 
 class IUploader(Interface):
-    def upload(uploadable):
+    def upload(uploadable, wait_for_numpeers=None):
         """Upload the file. 'uploadable' must impement IUploadable. This
         returns a Deferred which fires with the URI of the file."""
 
@@ -968,63 +1098,112 @@ class IUploader(Interface):
     def upload_filehandle(filehane):
         """Like upload(), but accepts an open filehandle."""
 
-class IVirtualDrive(Interface):
-    """I am a service that may be available to a client.
+class IChecker(Interface):
+    def check(uri_to_check):
+        """Accepts an IVerifierURI, and checks upon the health of its target.
 
-    Within any client program, this service can be retrieved by using
-    client.getService('vdrive').
-    """
+        For now, uri_to_check must be an IVerifierURI. In the future we
+        expect to relax that to be anything that can be adapted to
+        IVerifierURI (like read-only or read-write dirnode/filenode URIs).
 
-    def have_public_root():
-        """Return a Boolean, True if get_public_root() will work."""
-    def get_public_root():
-        """Get the public read-write directory root.
+        This returns a Deferred. For dirnodes, this fires with either True or
+        False (dirnodes are not distributed, so their health is a boolean).
 
-        This returns a Deferred that fires with an IDirectoryNode instance
-        corresponding to the global shared root directory."""
+        For filenodes, this fires with a tuple of (needed_shares,
+        total_shares, found_shares, sharemap). The first three are ints. The
+        basic health of the file is found_shares / needed_shares: if less
+        than 1.0, the file is unrecoverable.
 
+        The sharemap has a key for each sharenum. The value is a list of
+        (binary) nodeids who hold that share. If two shares are kept on the
+        same nodeid, they will fail as a pair, and overall reliability is
+        decreased.
 
-    def have_private_root():
-        """Return a Boolean, True if get_public_root() will work."""
-    def get_private_root():
-        """Get the private directory root.
-
-        This returns a Deferred that fires with an IDirectoryNode instance
-        corresponding to this client's private root directory."""
-
-    def get_node_at_path(path):
-        """Transform a path into an IDirectoryNode or IFileNode.
-
-        The path can either be a single string or a list of path-name
-        elements. The former is generated from the latter by using
-        .join('/'). If the first element of this list is '~', the rest will
-        be interpreted relative to the local user's private root directory.
-        Otherwse it will be interpreted relative to the global public root
-        directory. As a result, the following three values of 'path' are
-        equivalent::
-
-         '/dirname/foo.txt'
-         'dirname/foo.txt'
-         ['dirname', 'foo.txt']
-
-        This method returns a Deferred that fires with the node in question,
-        or errbacks with an IndexError if the target node could not be found.
+        The IChecker instance remembers the results of the check. By default,
+        these results are stashed in RAM (and are forgotten at shutdown). If
+        a file named 'checker_results.db' exists in the node's basedir, it is
+        used as a sqlite database of results, making them persistent across
+        runs. To start using this feature, just 'touch checker_results.db',
+        and the node will initialize it properly the next time it is started.
         """
 
-    def get_node(uri):
-        """Transform a URI (or IURI) into an IDirectoryNode or IFileNode.
+    def verify(uri_to_check):
+        """Accepts an IVerifierURI, and verifies the crypttext of the target.
 
-        This returns a Deferred that will fire with an instance that provides
-        either IDirectoryNode or IFileNode, as appropriate."""
+        This is a more-intensive form of checking. For verification, the
+        file's crypttext contents are retrieved, and the associated hash
+        checks are performed. If a storage server is holding a corrupted
+        share, verification will detect the problem, but checking will not.
+        This returns a Deferred that fires with True if the crypttext hashes
+        look good, and will probably raise an exception if anything goes
+        wrong.
 
-    def create_directory():
-        """Return a new IDirectoryNode that is empty and not linked by
-        anything."""
+        For dirnodes, 'verify' is the same as 'check', so the Deferred will
+        fire with True or False.
+
+        Verification currently only uses a minimal subset of peers, so a lot
+        of share corruption will not be caught by it. We expect to improve
+        this in the future.
+        """
+
+    def checker_results_for(uri_to_check):
+        """Accepts an IVerifierURI, and returns a list of previously recorded
+        checker results. This method performs no checking itself: it merely
+        reports the results of checks that have taken place in the past.
+
+        Each element of the list is a two-entry tuple: (when, results).
+        The 'when' values are timestamps (float seconds since epoch), and the
+        results are as defined in the check() method.
+
+        Note: at the moment, this is specified to return synchronously. We
+        might need to back away from this in the future.
+        """
+
+class IClient(Interface):
+    def upload(uploadable, wait_for_numpeers=None):
+        """Upload some data into a CHK, get back the URI string for it.
+        @param uploadable: something that implements IUploadable
+        @param wait_for_numpeers: don't upload anything until we have at least
+                                  this many peers connected
+        @return: a Deferred that fires with the (string) URI for this file.
+        """
+
+    def create_mutable_file(contents="", wait_for_numpeers=None):
+        """Create a new mutable file with contents, get back the URI string.
+        @param contents: the initial contents to place in the file.
+        @param wait_for_numpeers: don't upload anything until we have at least
+                                  this many peers connected
+        @return: a Deferred that fires with tne (string) SSK URI for the new
+                 file.
+        """
+
+    def create_empty_dirnode(wait_for_numpeers=None):
+        """Create a new dirnode, empty and unattached.
+        @param wait_for_numpeers: don't create anything until we have at least
+                                  this many peers connected.
+        @return: a Deferred that fires with the new IDirectoryNode instance.
+        """
+
+    def create_node_from_uri(uri):
+        """Create a new IFilesystemNode instance from the uri, synchronously.
+        @param uri: a string or IURI-providing instance. This could be for a
+                    LiteralFileNode, a CHK file node, a mutable file node, or
+                    a directory node
+        @return: an instance that provides IFilesystemNode (or more usefully one
+                 of its subclasses). File-specifying URIs will result in
+                 IFileNode or IMutableFileNode -providing instances, like
+                 FileNode, LiteralFileNode, or MutableFileNode.
+                 Directory-specifying URIs will result in
+                 IDirectoryNode-providing instances, like NewDirectoryNode.
+        """
 
 
 class NotCapableError(Exception):
     """You have tried to write to a read-only node."""
 
+class BadWriteEnablerError(Exception):
+    pass
+
 class RIControlClient(RemoteInterface):
 
     def wait_for_client_connections(num_clients=int):
@@ -1054,10 +1233,11 @@ class RIControlClient(RemoteInterface):
         measuring memory consupmtion in bytes."""
         return DictOf(str, int)
 
-    def speed_test(count=int, size=int):
+    def speed_test(count=int, size=int, mutable=bool):
         """Write 'count' tempfiles to disk, all of the given size. Measure
         how long (in seconds) it takes to upload them all to the servers.
-        Then measure how long it takes to download all of them.
+        Then measure how long it takes to download all of them. If 'mutable'
+        is True, use mutable files instead of immutable ones.
 
         Returns a tuple of (upload_time, download_time).
         """
