@@ -2,12 +2,12 @@
 
 import urllib
 
-def get(nodeurl, root_uri, vdrive_fname, local_file, stdout, stderr):
+def get(nodeurl, dir_uri, vdrive_fname, local_file, stdout, stderr):
     if nodeurl[-1] != "/":
         nodeurl += "/"
-    url = nodeurl + "uri/%s/" % urllib.quote(root_uri.replace("/","!"))
+    url = nodeurl + "uri/%s/" % urllib.quote(dir_uri)
     if vdrive_fname:
-        url += vdrive_fname
+        url += urllib.quote(vdrive_fname)
 
     if local_file is None or local_file == "-":
         outf = stdout
@@ -31,7 +31,7 @@ def main():
     import optparse, re
     parser = optparse.OptionParser()
     parser.add_option("-u", "--nodeurl", dest="nodeurl")
-    parser.add_option("-r", "--root-uri", dest="rooturi")
+    parser.add_option("-r", "--dir-cap", dest="rooturi")
 
     (options, args) = parser.parse_args()
 
@@ -40,8 +40,8 @@ def main():
         raise ValueError("--node-url is required to be a string and look like \"http://HOSTNAMEORADDR:PORT\", not: %r" % (options.nodeurl,))
 
     if not options.rooturi:
-        raise ValueError("must provide --root-uri")
-    
+        raise ValueError("must provide --dir-cap")
+
     vdrive_fname = args[0]
     local_file = None
     if len(args) > 1:
