@@ -3,7 +3,7 @@
 import urllib
 from allmydata.scripts.common_http import do_http
 
-def rm(nodeurl, root_uri, vdrive_pathname, verbosity, stdout, stderr):
+def rm(nodeurl, dir_uri, vdrive_pathname, verbosity, stdout, stderr):
     """
     @param verbosity: 0, 1, or 2, meaning quiet, verbose, or very verbose
 
@@ -11,9 +11,9 @@ def rm(nodeurl, root_uri, vdrive_pathname, verbosity, stdout, stderr):
     """
     if nodeurl[-1] != "/":
         nodeurl += "/"
-    url = nodeurl + "uri/%s/" % urllib.quote(root_uri.replace("/","!"))
+    url = nodeurl + "uri/%s/" % urllib.quote(dir_uri)
     if vdrive_pathname:
-        url += vdrive_pathname
+        url += urllib.quote(vdrive_pathname)
 
     resp = do_http("DELETE", url)
 
@@ -29,7 +29,7 @@ def main():
     import optparse, re
     parser = optparse.OptionParser()
     parser.add_option("-u", "--node-url", dest="nodeurl")
-    parser.add_option("-r", "--root-uri", dest="rooturi")
+    parser.add_option("-r", "--dir-cap", dest="rooturi")
 
     (options, args) = parser.parse_args()
 
@@ -38,8 +38,8 @@ def main():
         raise ValueError("--node-url is required to be a string and look like \"http://HOSTNAMEORADDR:PORT\", not: %r" % (options.nodeurl,))
 
     if not options.rooturi:
-        raise ValueError("must provide --root-uri")
-    
+        raise ValueError("must provide --dir-cap")
+
     vdrive_pathname = args[0]
 
     return rm(options.nodeurl, options.rooturi, vdrive_pathname, 0)
