@@ -14,6 +14,10 @@ class LostPeerError(Exception):
 def flip_bit(good): # flips the last bit
     return good[:-1] + chr(ord(good[-1]) ^ 0x01)
 
+class FakeClient:
+    def log(self, *args, **kwargs):
+        pass
+
 class FakeBucketWriterProxy:
     implements(IStorageBucketWriter, IStorageBucketReader)
     # these are used for both reading and writing
@@ -60,7 +64,7 @@ class FakeBucketWriterProxy:
             assert self.block_hashes is None
             self.block_hashes = blockhashes
         return defer.maybeDeferred(_try)
-        
+
     def put_share_hashes(self, sharehashes):
         def _try():
             assert not self.closed
@@ -218,7 +222,7 @@ class Encode(unittest.TestCase):
     # all tests encode to 100 shares, which means the share hash tree will
     # have 128 leaves, which means that buckets will be given an 8-long share
     # hash chain
-    
+
     # all 3-segment files will have a 4-leaf blockhashtree, and thus expect
     # to get 7 blockhashes. 4-segment files will also get 4-leaf block hash
     # trees and 7 blockhashes. 5-segment files will get 8-leaf block hash
@@ -244,10 +248,6 @@ class Encode(unittest.TestCase):
         # 4 segments: 25,25,25,25
         return self.do_encode(25, 100, 100, 4, 7, 8)
 
-    def test_send_101(self):
-        # encode a 101 byte file (in 5 segments: 25,25,25,25,1) to 100 shares
-        return self.do_encode(25, self.make_data(101), 100, 5, 15, 8)
-
     def test_send_124(self):
         # 5 segments: 25, 25, 25, 25, 24
         return self.do_encode(25, 124, 100, 5, 15, 8)
@@ -330,7 +330,7 @@ class Roundtrip(unittest.TestCase):
                            size=file_size)
         URI = u.to_string()
 
-        client = None
+        client = FakeClient()
         target = download.Data()
         fd = download.FileDownloader(client, URI, target)
 
