@@ -12,8 +12,9 @@ from allmydata.util.hashutil import file_renewal_secret_hash, \
      storage_index_chk_hash, plaintext_segment_hasher, key_hasher
 from allmydata import encode, storage, hashtree, uri
 from allmydata.util import idlib, mathutil
+from allmydata.util.assertutil import precondition
 from allmydata.interfaces import IUploadable, IUploader, IEncryptedUploadable
-from allmydata.Crypto.Cipher import AES
+from pycryptopp.cipher.aes import AES
 
 from cStringIO import StringIO
 
@@ -38,6 +39,8 @@ class PeerTracker:
                  sharesize, blocksize, num_segments, num_share_hashes,
                  storage_index,
                  bucket_renewal_secret, bucket_cancel_secret):
+        precondition(isinstance(peerid, str), peerid)
+        precondition(len(peerid) == 20, peerid)
         self.peerid = peerid
         self.permutedid = permutedid
         self.connection = connection # to an RIClient
@@ -49,7 +52,7 @@ class PeerTracker:
                                     num_share_hashes,
                                     EXTENSION_SIZE)
         self.allocated_size = as
-                                                           
+
         self.blocksize = blocksize
         self.num_segments = num_segments
         self.num_share_hashes = num_share_hashes
@@ -61,7 +64,7 @@ class PeerTracker:
 
     def __repr__(self):
         return ("<PeerTracker for peer %s and SI %s>"
-                % (idlib.b2a(self.peerid)[:4],
+                % (idlib.shortnodeid_b2a(self.peerid),
                    idlib.b2a(self.storage_index)[:6]))
 
     def query(self, sharenums):
@@ -84,7 +87,7 @@ class PeerTracker:
                                            canary=Referenceable())
         d.addCallback(self._got_reply)
         return d
-        
+
     def _got_reply(self, (alreadygot, buckets)):
         #log.msg("%s._got_reply(%s)" % (self, (alreadygot, buckets)))
         b = {}
@@ -329,7 +332,7 @@ class EncryptAnUploadable:
 
         d = self.original.get_encryption_key()
         def _got(key):
-            e = AES.new(key=key, mode=AES.MODE_CTR, counterstart="\x00"*16)
+            e = AES(key)
             self._encryptor = e
 
             storage_index = storage_index_chk_hash(key)
@@ -390,7 +393,7 @@ class EncryptAnUploadable:
                 chunk = data.pop(0)
                 self._plaintext_hasher.update(chunk)
                 self._update_segment_hash(chunk)
-                cryptdata.append(self._encryptor.encrypt(chunk))
+                cryptdata.append(self._encryptor.process(chunk))
                 del chunk
             return cryptdata
         d.addCallback(_got)
@@ -415,13 +418,21 @@ class EncryptAnUploadable:
 class CHKUploader:
     peer_selector_class = Tahoe2PeerSelector
 
-    def __init__(self, client, options={}):
+    def __init__(self, client, options={}, wait_for_numpeers=None):
+        assert wait_for_numpeers is None or isinstance(wait_for_numpeers, int), wait_for_numpeers
         self._client = client
+        self._wait_for_numpeers = wait_for_numpeers
         self._options = options
+        self._log_number = self._client.log("CHKUploader starting")
 
     def set_params(self, encoding_parameters):
         self._encoding_parameters = encoding_parameters
 
+    def log(self, msg, parent=None):
+        if parent is None:
+            parent = self._log_number
+        return self._client.log(msg, parent=parent)
+
     def start(self, uploadable):
         """Start uploading the file.
 
@@ -429,7 +440,7 @@ class CHKUploader:
         string)."""
 
         uploadable = IUploadable(uploadable)
-        log.msg("starting upload of %s" % uploadable)
+        self.log("starting upload of %s" % uploadable)
 
         eu = EncryptAnUploadable(uploadable)
         d = self.start_encrypted(eu)
@@ -443,9 +454,19 @@ class CHKUploader:
     def start_encrypted(self, encrypted):
         eu = IEncryptedUploadable(encrypted)
 
-        e = encode.Encoder(self._options)
+        e = encode.Encoder(self._options, self)
         e.set_params(self._encoding_parameters)
         d = e.set_encrypted_uploadable(eu)
+        def _wait_for_peers(res):
+            wait_for_numpeers = self._wait_for_numpeers
+            if wait_for_numpeers is None:
+                # wait_for_numpeers = e.get_param("share_counts")[0] # XXX
+                wait_for_numpeers = 1
+
+            d1 = self._client.introducer_client.when_enough_peers(wait_for_numpeers)
+            d1.addCallback(lambda dummy: res)
+            return d1
+        d.addCallback(_wait_for_peers)
         d.addCallback(self.locate_all_shareholders)
         d.addCallback(self.set_shareholders, e)
         d.addCallback(lambda res: e.start())
@@ -455,6 +476,7 @@ class CHKUploader:
     def locate_all_shareholders(self, encoder):
         storage_index = encoder.get_param("storage_index")
         upload_id = idlib.b2a(storage_index)[:6]
+        self.log("using storage index %s" % upload_id)
         peer_selector = self.peer_selector_class(upload_id)
 
         share_size = encoder.get_param("share_size")
@@ -472,7 +494,7 @@ class CHKUploader:
         """
         @param used_peers: a sequence of PeerTracker objects
         """
-        log.msg("_send_shares, used_peers is %s" % (used_peers,))
+        self.log("_send_shares, used_peers is %s" % (used_peers,))
         for peer in used_peers:
             assert isinstance(peer, PeerTracker)
         buckets = {}
@@ -511,7 +533,7 @@ def read_this_many_bytes(uploadable, size, prepend_data=[]):
 
 class LiteralUploader:
 
-    def __init__(self, client, options={}):
+    def __init__(self, client, wait_for_numpeers, options={}):
         self._client = client
         self._options = options
 
@@ -614,7 +636,8 @@ class Uploader(service.MultiService):
     # 'total' is the total number of shares created by encoding. If everybody
     # has room then this is is how many we will upload.
 
-    def upload(self, uploadable, options={}):
+    def upload(self, uploadable, options={}, wait_for_numpeers=None):
+        assert wait_for_numpeers is None or isinstance(wait_for_numpeers, int), wait_for_numpeers
         # this returns the URI
         assert self.parent
         assert self.running
@@ -628,7 +651,7 @@ class Uploader(service.MultiService):
             uploader_class = self.uploader_class
             if size <= self.URI_LIT_SIZE_THRESHOLD:
                 uploader_class = LiteralUploader
-            uploader = uploader_class(self.parent, options)
+            uploader = uploader_class(self.parent, options, wait_for_numpeers)
             uploader.set_params(self.parent.get_encoding_parameters()
                                 or self.DEFAULT_ENCODING_PARAMETERS)
             return uploader.start(uploadable)
